/* mbox.c -- mailbox status handling
 *
 * Copyright(C) 2001-2002 Elisa Manara <e@entropika.net>
 * This code is released under the GPL License version 2 */

#include <gtk/gtk.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include "melon.h"

static void mbox_set_active (char *mbox);
static void mbox_set_inactive (char *mbox);
static void register_row ( GtkWidget	*m_list,
			gint		row,
			GdkEventButton	*event );
static void remove_mbox ( GtkWidget *widget, gpointer user_data );
static void mbox_selector ( GtkWidget *widget, gpointer user_data );
static void add_mbox (GtkWidget *selector, gpointer user_data);
static void clear_entry(GtkWidget *widget, gpointer user_data);
static void set_mailer (GtkWidget *widget, gpointer user_data);
gint select_mbox (GtkWidget *widget, gpointer data);

static int sel_row;
GtkWidget *m_list, *filesel;
GtkWidget *entry;

extern char **environ;

gint check_mbox (gpointer data)
{
	int i;
	int retval;
	struct stat buf;
	time_t act_time;

	/* check mailbox and choice pixmap to show */
	time(&act_time);

	for (i = 0; i < MAX_MBOX; i++)
	{
		if (cfg_data.mbox[i][0] == '\0')
			break;
		if((retval = stat(cfg_data.mbox[i], &buf)) == -1) {
			char *error_string;
			error_string=strerror(errno);
			melon_error("Stat error for mailbox %s: %s\n", cfg_data.mbox[i], error_string);
			break;
		} 

		if (buf.st_ctime > w_data.see_time[i]) {
			/* config: play only one time */
			if (cfg_data.beep_mode == 0 && !w_data.beep[i]) {
				play_beep();
				mbox_set_active(cfg_data.mbox[i]);
			/* config: play for each new e-mail */
			} else if (cfg_data.beep_mode == 1 && buf.st_ctime > w_data.c_time[i]) {
				play_beep();
				if (!w_data.beep[i])
					mbox_set_active(cfg_data.mbox[i]);
				w_data.c_time[i] = buf.st_ctime;
			}

			w_data.beep[i] = 1;
			gtk_pixmap_set( GTK_PIXMAP(w_data.pixmap), 
				w_data.new_gdk_pixmap, w_data.mask2 );
			gtk_widget_shape_combine_mask( w_data.window, w_data.mask2, 0, 0 );
		} else if (data) {
			gtk_pixmap_set( GTK_PIXMAP(w_data.pixmap),
                                w_data.gdk_pixmap, w_data.mask1 );
		}
	} /* end of for */

	return 1;
}

#define ARGS_MAX 32
gint see_mail (GtkWidget *widget, char *mbox)
{
	int retval;
	int childpid;
	int i;
	char *tok;
	char *cmd_string;
	char *mailer_args[ARGS_MAX];
	char *path = NULL;

	/* TODO */
	if (!mbox)
		mbox = cfg_data.mbox[0];

	if(cfg_data.mailer[0] == '\0') {
		reset_mbox_status(mbox);
		return 1;
	}

	if ((childpid = fork()) == -1) {
		perror ("fork");
	}

	/* child */
	if (childpid == 0) {
		for(i = 0; i < ARGS_MAX; i++)
			mailer_args[i] = NULL;
			
		cmd_string = strdup(cfg_data.mailer);
		i = 0;
#ifdef __hpux
	while((tok = strtok(&cmd_string, " ")) != NULL)
#else
		while((tok = strsep(&cmd_string, " ")) != NULL)
#endif
		{
			if (i == ARGS_MAX) {
				melon_error("Too much arguments in mailer command line\n");
				exit(1);
			}
			if(tok[0] == '\0')
				continue;
			if (i == 0) {
				char *p = strrchr(tok, '/');
				path = strdup(tok);
				if (!p)
					mailer_args[i] = strdup(tok);
				else
					mailer_args[i] = strdup(p+1);
			} else {
				if(strcmp(tok, "%M") == 0) {
					mailer_args[i] = strdup(mbox);
				} else {
					mailer_args[i] = strdup(tok);
				}
			}
			i++;
		}
		if (i == 0) { exit(0); } 

		/*
		printf("path: %s\n", path);
		for (i = 0; mailer_args[i]; i++)
			printf("%s, ", mailer_args[i]);
		printf("\n");
		*/

		free(cmd_string);

		retval = execve (path, mailer_args, environ);
		for (i = 0; i < ARGS_MAX; i++)
			if (mailer_args[i])
				free(mailer_args[i]);
	
		if (retval == -1) {
			char *error_string;
			error_string=strerror(errno);
			melon_error("Cannot execute mailer program: %s\n", error_string);
			exit(0);
		}
		exit(1);
	} else {
		wait(NULL);
		reset_mbox_status(mbox);
	}

	return 1;
}

void reset_mbox_status (char *mbox)
{	
	struct stat buf;
	int i;
	int retval;

	for (i = 0; i < MAX_MBOX; i++)
	{
		if (cfg_data.mbox[i][0] == '\0')
			break;
		if (!strcmp(cfg_data.mbox[i], mbox)) {
			if((retval = stat(cfg_data.mbox[i], &buf)) == -1) {
				char msg[1024];
				snprintf(msg, 1024, 
				"Stat: error on reset status of %s", cfg_data.mbox[i]);
				return;
			}
			w_data.see_time[i] = buf.st_ctime;
			w_data.beep[i] = 0;
			break;
		}
	}

	mbox_set_inactive(mbox);
		
	/* reset icon if no one mbox is active */
	if(none_active_mbox()) {
		gtk_pixmap_set( GTK_PIXMAP(w_data.pixmap),
			w_data.gdk_pixmap, w_data.mask1 );
		gtk_widget_shape_combine_mask( w_data.window, w_data.mask1, 0, 0 );
	}
}

void mbox_set_active (char *mbox)
{
	int c;

	for(c = 0; c < MAX_MBOX; c++)
	{
		if(active_mbox[c][0] == '\0') {
			strncpy(active_mbox[c], mbox, MAX_PLEN);
			return;
		}
	}
}

void mbox_set_inactive (char *mbox)
{
	int c;

	for(c = 0; c < MAX_MBOX; c++)
	{
		if (!strcmp(active_mbox[c], mbox)) {
			active_mbox[c][0] = '\0';
			return;
		}
	}
}

int none_active_mbox(void)
{
	int c;

	for(c = 0; c < MAX_MBOX; c++)
	{
		if (active_mbox[c][0] != '\0') {
			return 0;
		}
	}

	return 1;
}

gint select_mbox (GtkWidget *widget, gpointer data)
{
	GtkWidget *sel_window;
	GtkWidget *vbox, *hbox;
	GtkWidget *scroll_win;
	GtkWidget *button;
	gchar *titles[1] = { "Mailboxes" };
	gchar *m_data[1];
	int c;

	/* window */
	sel_window = gtk_window_new( GTK_WINDOW_DIALOG);
	gtk_window_set_position(GTK_WINDOW (sel_window), GTK_WIN_POS_MOUSE);
	gtk_widget_set_usize( GTK_WIDGET (sel_window), 400, 240);
	gtk_window_set_title(GTK_WINDOW (sel_window), "Mailboxes");
	gtk_widget_show(sel_window);

	/* vbox */
	vbox = gtk_vbox_new( FALSE, 0 );
	gtk_container_add( GTK_CONTAINER( sel_window ), vbox );
	gtk_widget_show( vbox );

	/* scrolled window */
	scroll_win = gtk_scrolled_window_new( NULL, NULL );
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW( scroll_win ), 
		GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
	gtk_box_pack_start( GTK_BOX( vbox ), scroll_win, TRUE, TRUE, 0 );
	gtk_widget_show(scroll_win);

	/* clist */
	m_list = gtk_clist_new_with_titles( 1, titles );

	gtk_signal_connect(GTK_OBJECT(m_list), "select_row",
		GTK_SIGNAL_FUNC(register_row), NULL);

	gtk_widget_set_usize( GTK_WIDGET (m_list), 200, 200);
	gtk_clist_set_row_height( GTK_CLIST (m_list), 20);
	gtk_clist_column_titles_passive( GTK_CLIST( m_list ));
	gtk_clist_set_shadow_type (GTK_CLIST(m_list), GTK_SHADOW_OUT);
	gtk_clist_set_selection_mode( GTK_CLIST( m_list ),
		GTK_SELECTION_SINGLE);

	//gtk_clist_set_column_width(GTK_CLIST( m_list ), 0, 25);

	gtk_container_add( GTK_CONTAINER( scroll_win ), m_list );

	for (c = 0; c < MAX_MBOX; c++)
	{
		if(cfg_data.mbox[c][0] == '\0')
			continue;
		m_data[0] = cfg_data.mbox[c];
		gtk_clist_append(GTK_CLIST (m_list), m_data);
	}

        gtk_widget_show(m_list);

	/* hbox */
	hbox = gtk_hbox_new( FALSE, 0 );
	//gtk_box_pack_start( GTK_BOX( vbox ), scroll_win, FALSE, FALSE, 0 );
	gtk_container_add( GTK_CONTAINER( vbox ), hbox);
	gtk_widget_show( hbox );

        /* REMOVE button */
        button = gtk_button_new_with_label ("Remove");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                GTK_SIGNAL_FUNC(remove_mbox),
                GTK_OBJECT (m_list));
        gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
        gtk_widget_show (button);

        /* ADD NEW button */
        button = gtk_button_new_with_label ("Add New");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                GTK_SIGNAL_FUNC(mbox_selector),
                GTK_OBJECT (m_list));
        gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
        gtk_widget_show (button);

        /* CLOSE button */
        button = gtk_button_new_with_label ("Close");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                GTK_SIGNAL_FUNC(gtk_widget_destroy),
                GTK_OBJECT (sel_window));
        gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
        gtk_widget_show (button);

        return 1;
}

void register_row ( GtkWidget	*widget,
                gint            row,
                GdkEventButton  *event )
{
	sel_row = row;
	return;
}

void remove_mbox ( GtkWidget *widget, gpointer user_data )
{
	int c, i;
	char *row_data;

	gtk_clist_get_text (GTK_CLIST( m_list ), sel_row, 0, &row_data);

	for (c = 0; c < MAX_MBOX; c++)
	{
		if (!strcmp(cfg_data.mbox[c], row_data)) {
			cfg_data.mbox[c][0] = '\0';
			break;
		}
	}

	gtk_clist_remove(GTK_CLIST ( m_list ), sel_row);

	/* reorder cfg_data.mbox array */
	for (i = MAX_MBOX -1; i > c; i--)
	{
		if (cfg_data.mbox[i] != '\0') {
			strncpy(cfg_data.mbox[c], cfg_data.mbox[i], MAX_PLEN);
			cfg_data.mbox[i][0] = '\0';
			break;
		}
	}
	/* update melon.cfg file for next startup */
	config_update();
	
	return;
}

void mbox_selector ( GtkWidget *widget, gpointer user_data )
{
	filesel = gtk_file_selection_new ("Add or Remove Mailbox");
	gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (filesel));


	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(filesel)->ok_button),
		"clicked", GTK_SIGNAL_FUNC (add_mbox), NULL);
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(filesel)->ok_button),
		"clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
		(gpointer) filesel);
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(filesel)->cancel_button),
		"clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
		(gpointer) filesel);

	gtk_widget_show (filesel);

        return;
}

void add_mbox (GtkWidget *selector, gpointer user_data)
{
	char *mbox;
	int i;
	gchar *m_data[1];
	struct stat buf;

	mbox = gtk_file_selection_get_filename (GTK_FILE_SELECTION(filesel));

	for (i = 0; i <= MAX_MBOX; i++)
	{
		if (i == MAX_MBOX) {
			dialog_message (" Max mboxes number reached ");
			return;
		}
		if(cfg_data.mbox[i][0] == '\0') {
			strncpy(cfg_data.mbox[i], mbox, MAX_PLEN);
			break;
		}
	}

	m_data[0] = cfg_data.mbox[i];
	gtk_clist_append(GTK_CLIST (m_list), m_data);

	/* initialize mbox status */
	stat(cfg_data.mbox[i], &buf);
	w_data.see_time[i] = buf.st_ctime;
	w_data.beep[i] = 0;
	
	/* update melon.cfg file for next startup */
	config_update();
	
	return;
}

gint mailer_config (GtkWidget *widget, gpointer data)
{
	GtkWidget *window;
	GtkWidget *vbox, *hbox2;
	GtkWidget *button, *label;

	/* window */
	window = gtk_window_new( GTK_WINDOW_DIALOG);
	gtk_window_set_position(GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
	gtk_widget_set_usize( GTK_WIDGET (window), 200, 70);
	gtk_window_set_title(GTK_WINDOW (window), "Set Icon Position");
	gtk_widget_show(window);

	/* vbox */
	vbox = gtk_vbox_new( FALSE, 0 );
	gtk_container_add( GTK_CONTAINER( window ), vbox );
	gtk_widget_show( vbox );

	/* label */
	label = gtk_label_new ("Command line: ");
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 2);
	gtk_widget_show(label);

	/* entry */
	entry = gtk_entry_new();
	gtk_entry_set_max_length (GTK_ENTRY( entry ), MAX_PLEN);

	if(cfg_data.mailer[0] != '\0')
		gtk_entry_set_text (GTK_ENTRY( entry ), cfg_data.mailer);
	gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, FALSE, 2);
	gtk_widget_show(entry);

	/* hbox2 */
	hbox2 = gtk_hbox_new( FALSE, 0 );
	gtk_container_add( GTK_CONTAINER( vbox ), hbox2);
	gtk_widget_show( hbox2 );

	/* APPLY button */
	button = gtk_button_new_with_label ("Apply");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC(set_mailer), NULL);
	gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	/* CLEAR button */
	button = gtk_button_new_with_label ("Clear");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC(clear_entry), NULL);
	gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	/* CLOSE button */
	button = gtk_button_new_with_label ("Close");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy),
		GTK_OBJECT (window));
	gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	return 1;

}

void clear_entry(GtkWidget *widget, gpointer user_data)
{
	gtk_entry_set_text(GTK_ENTRY( entry ), "");
	return;
}

void set_mailer(GtkWidget *widget, gpointer user_data)
{
	char *text;

	text = gtk_editable_get_chars(GTK_EDITABLE( entry ), 0, -1);
	if (text[0] != '\0') {
		strncpy(cfg_data.mailer, text, MAX_PLEN);
	} else {
		cfg_data.mailer[0] = '\0';
	}

	g_free(text);

	dialog_message (" Mailer command configured ");
	config_update();

	return;
}
